/**
 * FastReverseDNS batch reverse DNS lookup
 * Copyright (C) 2002 Jesse Wilson <jesse@swank.ca>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.swank.fastreversedns;

/**
 * performs DNS lookups avoiding idle blocking by using multiple threads
 */
public class FastReverseDNS implements Runnable
{
  // the reverse DNS service to use
  private ReverseDNS dns;
  // the IP addresses to traverse
  private IPEnumeration IPList;
  // the default number of threads to use in normal use
  private static final int DEFAULT_NUM_THREADS = 60;


  /**
   * Creates a new instance that will query IP addresses from the specified
   * list on the specified ReverseDNS server.
   */
  public FastReverseDNS(ReverseDNS dns, IPEnumeration IPList)
  { this.dns = dns;
    this.IPList = IPList;
  }


  /**
   * While IPs are left in the range, query the reverse server and output the
   * hostname
   */
  public void run()
  { boolean moreIPs = true;

    while(moreIPs)
    { String currentIP;

      // attempt to get a new unique IP from the list
      synchronized(IPList)
      { if(IPList.hasMoreElements())
        { currentIP = (String)IPList.nextElement();
        } else
        { moreIPs = false;
          break;
        }
      }

      // get the hostname for this IP
      String hostname = dns.doReverseLookup(currentIP);

      // don't display hostnames that are null
      if(hostname != null)
      { // display this hostname to system.out
        synchronized(System.out)
        { System.out.println(currentIP + "\t" + hostname);
        }
      }
    }
  }

  /**
   * Displays a set of IP/hostname pairs using reverse DNS on the specified
   * list of hostnames.
   */
  public static void main(String[] args)
  { if(args.length != 3)
    { System.out.println("usage:   FastReverseDNS dns firstIP lastIP");
      System.out.println("example: FastReversesDNS 172.16.38.122 10.1.3.0 10.1.4.127");
      System.out.println("");
      System.out.println("CAUTION: Do not use FastReverseDNS in an environment where");
      System.out.println("         you are not authorized. This software will hammer");
      System.out.println("         on your DNS server and may trigger a security alert!");
      System.exit(1);
    }

    ReverseDNS dns = new ReverseDNS(args[0]);
    IPEnumeration ips = new IPEnumeration(args[1], args[2]);

    // for each threads, go through and perform DNS queries
    int threads = DEFAULT_NUM_THREADS;
    for(int i = 0; i < threads; i++)
    { FastReverseDNS hostnameLookupThread = new FastReverseDNS(dns, ips);
      new Thread(hostnameLookupThread).start();
    }
  }
}